/* multiboot_header.S - the Multiboot header */
/* Copyright (C) 1999, 2001  J. Geerts.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#define ASM 1
#include "multiboot_header.h"

.section .bss
.align 4096
BootPageDirectory:
  /* This page directory will map the first four megabytes of 32-bit physical
     address space. All bits are clear except the following:
     bit 7: PS The kernel page is 4MB
     bit 1: RW The kernel page is read/write
     bit 0: P  The kernel page is present
     The page directory is filled in using code to reduce the kernel's file size */
  .fill (1024), 4, 0

/* make sure the '.multiboot.header' section starts here */
.section .multiboot.header
.align 4
.long MULTIBOOT_HEADER_MAGIC
.long MULTIBOOT_HEADER_FLAGS
.long MULTIBOOT_HEADER_MAGIC | MULTIBOOT_HEADER_FLAGS

/* reserve initial kernel stack space */
.comm stack, STACK_SIZE, 32               # reserve 16K stack on a quadword boundary

.global start; start:
  /* reset EFLAGS */
  pushl   $0
  popf

  /* fill in the Page Directory entries 0 and KERNEL_PAGE_NUMBER */
  mov     $0x00000083, %eax               # PDE flags
  mov     %eax, (BootPageDirectory - KERNEL_VIRTUAL_BASE)
  mov     %eax, (BootPageDirectory + KERNEL_PAGE_OFFSET - KERNEL_VIRTUAL_BASE)

  /* IMPORTANT: Until paging is set up, the code must be position-independent
     and use the physical addresses, not the virtual ones. */
  mov     $(BootPageDirectory - KERNEL_VIRTUAL_BASE), %ecx
  mov     %ecx, %cr3

  /* enable 4MB pages */
  mov     %cr4, %ecx
  or      $0x00000010, %ecx
  mov     %ecx, %cr4

  /* enable paging */
  mov     %cr0, %ecx
  or      $0x80000000, %ecx
  mov     %ecx, %cr0

  /* continue fetching instructions from kernel space */
  lea     ContinueInHigherHalf, %ecx
  jmp     *%ecx

ContinueInHigherHalf:
  /* Done, paging enabled and first page unmapped. The first 4MB of physical
     address space is mapped starting at KERNEL_VIRTUAL_BASE. Everything is
     linked to this address, so no more position-independent code or address
     conversions are necessary. */
  mov     $(stack + STACK_SIZE), %esp     # set up a stack

  push    %eax                            # Multiboot magic number
  push    %ebx                            # Multiboot data structure
  call    k_main                          # call kernel

  cli                                     # clear IF so only special interrupts can interrupt us
halt:
  hlt                                     # halt the CPU
  jmp     halt                            # jump back in case of an interrupt
